@@ -40,6 +40,11 @@
 #include "misc.h"
 
 
+#ifndef DISABLE_GNOME
+#include <gnome.h>
+#endif
+
+#include <gconf/gconf-client.h>
 /* Declarations */
 extern GnomeMeeting *MyApp;
 extern GtkWidget *gm;
@@ -51,6 +56,7 @@ GMThreadsCleaner::GMThreadsCleaner ()
   :PThread (1000, AutoDeleteThread)
 {
   gw = gnomemeeting_get_main_window (gm);
+  gw->cleaner_thread_count = 1;
 
   this->Resume ();
 }
@@ -58,55 +64,70 @@ GMThreadsCleaner::GMThreadsCleaner ()
 
 GMThreadsCleaner::~GMThreadsCleaner ()
 {
-  /* Nothing to do here */
+  gw->cleaner_thread_count = 0;
 }
 
 
 void GMThreadsCleaner::Main ()
 {
-  int counter = 0;
-
-  GMILSClient *ils_client = (GMILSClient *) 
-    MyApp->Endpoint ()->GetILSClient();
-
-  GMVideoGrabber *video_grabber = (GMVideoGrabber *) 
-    MyApp->Endpoint ()->GetVideoGrabber ();
+  gint timeout;
 
+  int x = 0, y = 0;
   GMH323EndPoint *endpoint = MyApp->Endpoint ();
 
-  gnomemeeting_threads_enter ();
-
-  gnome_appbar_push (GNOME_APPBAR (gw->statusbar), _("Quit in progress..."));
-
-
-  /* Synchronous End of Call */
-  endpoint->ClearAllCalls (H323Connection::EndedByLocalUser, FALSE);
-
-  gnomemeeting_threads_leave ();
-
-  while (MyApp->Endpoint ()->GetCallingState () != 0) {
-   
-    /* if OpenH323 doesn't disconnect, we force the exit */
-    if (counter > 50000) {
-      
-      gdk_threads_enter ();
-      gnomemeeting_warning_dialog (GTK_WINDOW (gm), _("Warning! GnomeMeeting can't disconnect from the remote party due to a bug in OpenH323.\nForcing the exit in 3 seconds!\n"));
-      gdk_threads_leave ();
-      PThread::Current()->Sleep (3000);
-
-      system ("killall -9 gnomemeeting");
-
-      break;
-    }
-
-    Current ()->Sleep (100);
-    counter++;  
-  }
-
-  delete (ils_client);
-  delete (video_grabber);
+  quit_mutex.Wait ();
   
   gnomemeeting_threads_enter ();
-  gtk_main_quit ();
+
+  gnomemeeting_statusbar_push (gw->statusbar, _("Quit in progress..."));
+
+  timeout = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (gm), "timeout"));
+  gtk_timeout_remove (timeout);
+
+  /* Synchronous End of Call */
+  if (endpoint->GetCallingState () != 0)
+    endpoint->ClearCall (endpoint->GetCurrentCallToken (), 
+			 H323Connection::EndedByLocalUser);
+
+  gtk_widget_hide_all (gm);
+  gtk_widget_hide_all (gw->pref_window);
+  gtk_widget_hide_all (gw->ldap_window);
+  gtk_widget_hide_all (gw->calls_history_window);
+  gtk_widget_hide_all (gw->history_window);
   gnomemeeting_threads_leave ();
+
+  while (endpoint->GetCallingState ())
+    Current ()->Sleep (50);
+
+  delete (endpoint);
+  gnomemeeting_threads_enter ();
+
+  gtk_window_get_size (GTK_WINDOW (gw->local_video_window), &x, &y);
+  gconf_client_set_int (gconf_client_get_default (), 
+			"/apps/gnomemeeting/video_display/local_video_width",
+			x, NULL);
+  gconf_client_set_int (gconf_client_get_default (), 
+			"/apps/gnomemeeting/video_display/local_video_height", 
+			y, NULL);
+
+  gtk_window_get_size (GTK_WINDOW (gw->remote_video_window), &x, &y);
+  gconf_client_set_int (gconf_client_get_default (), 
+			"/apps/gnomemeeting/video_display/remote_video_width",			x, NULL);
+  gconf_client_set_int (gconf_client_get_default (), 
+			"/apps/gnomemeeting/video_display/remote_video_height",
+			y, NULL);
+
+  gtk_main_quit ();
+
+  gnomemeeting_threads_leave ();
+
+  quit_mutex.Signal ();
+}
+
+
+void GMThreadsCleaner::Wait ()
+{
+  quit_mutex.Wait ();
+
+  quit_mutex.Signal ();
 }
