@@ -34,6 +34,10 @@
 #include "gnomemeeting.h"
 #include "misc.h"
 
+#ifndef DISABLE_GNOME
+#include <gnome.h>
+#endif
+
 
 /* Declarations */
 extern GnomeMeeting *MyApp;
@@ -60,7 +64,7 @@ void GMH323Gatekeeper::Main ()
   GtkWidget *msg_box = NULL;
   gchar *msg = NULL;
   gchar *gconf_string = NULL;
-  GConfClient *client = gconf_client_get_default ();
+  GConfClient *client = NULL;
   H323EndPoint *endpoint = NULL;
   GmPrefWindow *pw = NULL;
   GmWindow *gw = NULL;
@@ -68,12 +72,20 @@ void GMH323Gatekeeper::Main ()
   int method;
 
   /* Register using the gatekeeper host */
-  method = gconf_client_get_int (GCONF_CLIENT (client),
-				  "/apps/gnomemeeting/gatekeeper/registering_method", 0);
+  gnomemeeting_threads_enter ();
+  client = gconf_client_get_default ();
+  method =
+    gconf_client_get_int (GCONF_CLIENT (client),
+			  "/apps/gnomemeeting/gatekeeper/registering_method",
+			  0);
+  gnomemeeting_threads_leave ();
 
   endpoint = (H323EndPoint *) MyApp->Endpoint ();
 
+  gnomemeeting_threads_enter ();
   gconf_string = gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/gk_password", 0);
+  gnomemeeting_threads_leave ();
+
   endpoint->SetGatekeeperPassword ("");
   if ((gconf_string != NULL)&&(strcmp ("", gconf_string))) {
 
@@ -82,21 +94,7 @@ void GMH323Gatekeeper::Main ()
   g_free (gconf_string);
   gconf_string = NULL;
 
-
-  gconf_string = gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/gk_alias", 0);
-
-  /* set the alias to make sure it is set */
-  if ((gconf_string != NULL)&&(strcmp ("", gconf_string))) {
-
-    /* Remove the old aliases */
-    for (int i = endpoint->GetAliasNames ().GetSize () - 1; i >= 1; i--) 
-      if (!endpoint->GetAliasNames () [i].IsEmpty ())
-	endpoint->RemoveAliasName (endpoint->GetAliasNames () [i]);
-
-    endpoint->AddAliasName (gconf_string);
-  }
-  g_free (gconf_string);
-  gconf_string = NULL;
+  MyApp->Endpoint ()->SetUserNameAndAlias ();
 
 
   /* Fetch the needed data */
@@ -109,13 +107,15 @@ void GMH323Gatekeeper::Main ()
   /* Use the hostname */
   if (method == 1) {
 
+    gnomemeeting_threads_enter ();
     gconf_string = gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/gk_host", 0);
+    gnomemeeting_threads_leave ();
 
     if ((gconf_string == NULL) || (!strcmp ("", gconf_string))) {
      
       gnomemeeting_threads_enter ();
       msg_box = 
-	gtk_message_dialog_new (GTK_WINDOW (gw->pref_window),
+	gtk_message_dialog_new (GTK_WINDOW (gm),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CLOSE,
@@ -126,16 +126,18 @@ void GMH323Gatekeeper::Main ()
 				G_CALLBACK (gtk_widget_destroy),
 				GTK_OBJECT (msg_box));
 
-      gnomemeeting_threads_leave ();
-
       gconf_client_set_int (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/registering_method", 0, NULL);
+  
+      /* We disable microtelco if it was enabled */
+      gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
+			     false, 0);
+      gnomemeeting_threads_leave ();
 
       return;
     }
 
-    H323TransportUDP *ras_channel = new H323TransportUDP (*MyApp->Endpoint ());
-      
-    if (MyApp->Endpoint ()->SetGatekeeper(PString (gconf_string), ras_channel)) {
+
+    if (MyApp->Endpoint ()->SetGatekeeper(PString (gconf_string))) {
  
       msg = g_strdup_printf (_("Gatekeeper set to %s"), 
 			     (const char*) MyApp->Endpoint ()
@@ -143,9 +145,18 @@ void GMH323Gatekeeper::Main ()
 	  
       gnomemeeting_threads_enter ();
       gnomemeeting_log_insert (gw->history_text_view, msg);
-      gnomemeeting_statusbar_flash (gm, msg);
+      gnomemeeting_statusbar_flash (gw->statusbar, msg);
+
+      /* If the host to which we registered is the MicroTelco GK, enable
+	 MicroTelco, if not disable it, in case it was enabled */
+      if (PString (gconf_string).Find ("gk.microtelco.com") != P_MAX_INDEX)
+	gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
+			       true, 0);
+      else
+	gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
+			       false, 0);
       gnomemeeting_threads_leave ();
-      
+
       g_free (msg);
     } 
     else {
@@ -154,8 +165,9 @@ void GMH323Gatekeeper::Main ()
 			     gconf_string);
       
       gnomemeeting_threads_enter ();
+      gnomemeeting_log_insert (gw->history_text_view, msg);
       msg_box = 
-	gtk_message_dialog_new (GTK_WINDOW (gw->pref_window),
+	gtk_message_dialog_new (GTK_WINDOW (gm),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CLOSE,
@@ -165,9 +177,14 @@ void GMH323Gatekeeper::Main ()
       g_signal_connect_swapped (GTK_OBJECT (msg_box), "response",
 				G_CALLBACK (gtk_widget_destroy),
 				GTK_OBJECT (msg_box));
-
-      gnomemeeting_threads_leave ();
       
+      gconf_client_set_int (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/registering_method", 0, NULL);
+
+      /* We disable microtelco if it was enabled */
+      gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
+			     false, 0);
+      gnomemeeting_threads_leave ();
+
       g_free (msg);
     }
 
@@ -178,13 +195,18 @@ void GMH323Gatekeeper::Main ()
   /* Register using the gatekeeper ID */
   if (method == 2) {
 
+    /* We disable microtelco if it was enabled */
+    gnomemeeting_threads_enter ();
+    gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
+			   false, 0);
     gconf_string = gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/gk_id", 0);
+    gnomemeeting_threads_leave ();
 
     if ((gconf_string == NULL) || (!strcmp ("", gconf_string))) {
      
       gnomemeeting_threads_enter ();
       msg_box = 
-	gtk_message_dialog_new (GTK_WINDOW (gw->pref_window),
+	gtk_message_dialog_new (GTK_WINDOW (gm),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CLOSE,
@@ -195,9 +217,8 @@ void GMH323Gatekeeper::Main ()
 				G_CALLBACK (gtk_widget_destroy),
 				GTK_OBJECT (msg_box));
     
-      gnomemeeting_threads_leave ();
-
       gconf_client_set_int (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/registering_method", 0, NULL);
+      gnomemeeting_threads_leave ();
 
       return;
     }
@@ -211,7 +232,7 @@ void GMH323Gatekeeper::Main ()
 
       gnomemeeting_threads_enter ();
       gnomemeeting_log_insert (gw->history_text_view, msg);
-      gnomemeeting_statusbar_flash (gm, msg);
+      gnomemeeting_statusbar_flash (gw->statusbar, msg);
       gnomemeeting_threads_leave ();
       
       g_free (msg);
@@ -222,7 +243,7 @@ void GMH323Gatekeeper::Main ()
       
       gnomemeeting_threads_enter ();
       msg_box = 
-	gtk_message_dialog_new (GTK_WINDOW (gw->pref_window),
+	gtk_message_dialog_new (GTK_WINDOW (gm),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CLOSE,
@@ -233,8 +254,9 @@ void GMH323Gatekeeper::Main ()
 				G_CALLBACK (gtk_widget_destroy),
 				GTK_OBJECT (msg_box));
   
+      gconf_client_set_int (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/registering_method", 0, NULL);
       gnomemeeting_threads_leave ();
-      
+
       g_free (msg);
     }
   }
@@ -243,8 +265,13 @@ void GMH323Gatekeeper::Main ()
   /* Register after trying to discover the Gatekeeper */
   if (method == 3) {
 
-    if (MyApp->Endpoint ()
-	->DiscoverGatekeeper (new H323TransportUDP (*MyApp->Endpoint ()))) {
+    /* We disable microtelco if it was enabled */
+    gnomemeeting_threads_enter ();
+    gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
+			   false, 0);
+    gnomemeeting_threads_leave ();
+
+    if (MyApp->Endpoint ()->DiscoverGatekeeper ()) {
  
       msg = g_strdup_printf (_("Gatekeeper set to %s"), 
 			     (const char*) MyApp->Endpoint ()
@@ -252,7 +279,7 @@ void GMH323Gatekeeper::Main ()
 
       gnomemeeting_threads_enter ();
       gnomemeeting_log_insert (gw->history_text_view, msg);
-      gnomemeeting_statusbar_flash (gm, msg);
+      gnomemeeting_statusbar_flash (gw->statusbar, msg);
       gnomemeeting_threads_leave ();
       
       g_free (msg);
@@ -261,7 +288,7 @@ void GMH323Gatekeeper::Main ()
 
       gnomemeeting_threads_enter ();
       msg_box = 
-	gtk_message_dialog_new (GTK_WINDOW (gw->pref_window),
+	gtk_message_dialog_new (GTK_WINDOW (gm),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CLOSE,
@@ -272,7 +299,16 @@ void GMH323Gatekeeper::Main ()
 				G_CALLBACK (gtk_widget_destroy),
 				GTK_OBJECT (msg_box));
   
+      gconf_client_set_int (GCONF_CLIENT (client), "/apps/gnomemeeting/gatekeeper/registering_method", 0, NULL);
       gnomemeeting_threads_leave ();
+
     }
   }
+
+  gnomemeeting_threads_enter ();
+  if (method == 0)
+    /* We disable microtelco if it was enabled */
+    gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
+			   false, 0);
+  gnomemeeting_threads_leave ();
 }
